<?php

/**
 * 钩子中心
 * Class Hook
 */
class Hook
{
    //注册后保存下来
    private static $hooks = [];

    //注册插件
    public static function add($hook_name, $class)
    {
        $hook_name = strtolower($hook_name);
        if (isset(self::$hooks[$hook_name])) {
            self::$hooks[$hook_name] = array_merge(self::$hooks[$hook_name], $class);
        } else {
            self::$hooks[$hook_name][] = $class;
        }
        self::$hooks[$hook_name] = array_unique(self::$hooks[$hook_name]);
    }

    //批量导入
    public static function batch_add($plugins = [])
    {
        if (!empty($plugins)) {
            //以新的key覆盖旧的hook
            self::$hooks = $plugins + self::$hooks;

            //去重
            if (self::$hooks){
                foreach (self::$hooks as $hook=>&$class){
                    $class = array_unique($class);
                }
                unset($class);
            }
        }
    }

    //获取插件值
    public static function get($hook_name)
    {
        $hook_name = strtolower($hook_name);
        if (!isset(self::$hooks[$hook_name])) {
            return false;
        }
        return self::$hooks[$hook_name];
    }

    //监听
    public static function listen($hook_name, &$param = null)
    {
        $hook_name = strtolower($hook_name);
        if (!self::get($hook_name)) {
            return false;
        }

        $result = [];
        foreach (self::get($hook_name) as $key => $class) {
            $result[$key] = static::exec($class, !empty($param['method']) ? $param['method'] : 'main', $param);
            if ($result[$key] === false) {
                break;
            }
        }
        return $result;
    }

    //执行插件行为
    public static function exec($class, $function = 'main', $param = null)
    {
        if (is_object($class)) {
            if (!is_callable([$class, $function])) {
                $ret = false;
            } else {
                $ret = $class->$function($param);
            }
        } else {
            if (!class_exists($class)){
                return false;
            }
            $obj = new $class();

            if (!is_callable([$class, $function])) {
                $ret = false;
            } else {

                $ret = $obj->$function($param);
            }
        }
        return $ret;
    }

}